//
//  DefaultTemplate.swift
//  seguecode
//
//  Created by Ian Grossberg on 8/21/15.
//  Copyright (c) 2015 Adorkable. All rights reserved.
//

import Foundation

import Stencil

class DefaultTemplate: Template {
    struct Keys
    {
        static let FileName = "FileName"
        static let ProjectName = "ProjectName"
        
        static let GeneratedOn = "GeneratedOn"
        
        static let SharedDefinitions = "SharedDefinitions"
        
        static let ViewControllers = "ViewControllers"
        struct ViewController {
            static let Name = "ViewControllerName"
            
            static let SegueCases = "SegueCases"
            struct SegueCase
            {
                static let Identifier = "Identifier"
                static let SourceIdentifier = "SourceIdentifier"
                static let DestinationIdentifier = "DestinationIdentifier"
                static let Value = "Value"
            }
            
            static let PerformFunctions = "PerformFunctions"
            struct PerformFunction
            {
                static let Identifier = "Identifier"
                static let SourceIdentifier = "SourceIdentifier"
                static let DestinationIdentifier = "DestinationIdentifier"
            }
            
            static let TableViewCellPrototypes = "TableViewCellPrototypes"
            struct TableViewCellPrototype
            {
                static let ReuseIdentifier = "ReuseIdentifier"
            }
            
            static let DequeueFunctions = "DequeueFunctions"
            struct DequeueFunction
            {
                static let ReuseIdentifier = "ReuseIdentifier"
            }
        }
    }
    
    static var topFileComments : String {
        return "//\n" +
        "{% if \(Keys.FileName) %}" +
        "//  {{ \(Keys.FileName) }}\n" +
        "{% endif %}" +
            
        "{% if \(Keys.ProjectName) %}" +
        "//  {{ \(Keys.ProjectName) }}\n" +
        "{% endif %}" +
            
        "//\n" +
        "//  Generated by seguecode [http://bit.ly/seguecode]" +
        "{% if \(Keys.GeneratedOn) %}" +
        " on {{ \(Keys.GeneratedOn) }}" +
        "{% endif %}" +
        "\n" +
        // TODO: copywrite and company info
        "//"
    }

    // TODO: no longer cases, rename
    static var segueCases : String {
        return "{% if viewController.\(Keys.ViewController.SegueCases) %}" +
            
            "\n" +
            "    struct Segues {\n" +
            "{% for segueCase in viewController.\(Keys.ViewController.SegueCases) %}" +
            
            "        static let {{ segueCase.\(Keys.ViewController.SegueCase.SourceIdentifier) }}{{ segueCase.\(Keys.ViewController.SegueCase.Identifier) }}{{ segueCase.\(Keys.ViewController.SegueCase.DestinationIdentifier) }} = Segue(identifier: \"{{ segueCase.\(Keys.ViewController.SegueCase.Value) }}\")\n" +

            "{% endfor %}" +
            "    }\n" +
            
            "{% endif %}"
    }
    
    static var performFunctions : String {
        return "{% if viewController.\(Keys.ViewController.PerformFunctions) %}" +
            "{% for performFunction in viewController.\(Keys.ViewController.PerformFunctions) %}" +
            
            "\n" +
            "    @IBAction func perform{{ performFunction.\(Keys.ViewController.PerformFunction.SourceIdentifier) }}{{ performFunction.\(Keys.ViewController.PerformFunction.Identifier) }}{{ performFunction.\(Keys.ViewController.PerformFunction.DestinationIdentifier) }}(sender : AnyObject? = nil) {\n" +
            "        self.performSegue({{ viewController.\(Keys.ViewController.Name) }}.Segues.{{ performFunction.\(Keys.ViewController.PerformFunction.SourceIdentifier) }}{{ performFunction.\(Keys.ViewController.PerformFunction.Identifier) }}{{ performFunction.\(Keys.ViewController.PerformFunction.DestinationIdentifier) }}, sender: sender)\n" +
            "    }\n" +

            "{% endfor %}" +
            "{% endif %}"
    }
    
    static var tableViewCellPrototypes : String {
        return "{% if viewController.\(Keys.ViewController.TableViewCellPrototypes) %}" +
            
        "\n" +
        "    struct TableViewCellPrototypes {\n" +
        "{% for cellPrototype in viewController.\(Keys.ViewController.TableViewCellPrototypes) %}" +
            
        "        static let {{ cellPrototype.\(Keys.ViewController.TableViewCellPrototype.ReuseIdentifier) }} = UITableView.TableViewCellPrototype(reuseIdentifier: \"{{ cellPrototype.\(Keys.ViewController.TableViewCellPrototype.ReuseIdentifier) }}\")\n" +
        
        "{% endfor %}" +
        "    }\n" +
            
        "{% endif %}"
    }
    
    static var dequeueFunctions : String {
        return "{% if viewController.\(Keys.ViewController.DequeueFunctions) %}" +
            "{% for dequeueFunction in viewController.\(Keys.ViewController.DequeueFunctions) %}" +
            
            "\n" +
            "    func dequeueReusable{{ dequeueFunction.\(Keys.ViewController.DequeueFunction.ReuseIdentifier) }}(tableView : UITableView) -> AnyObject? {\n" +
            
            "        return tableView.dequeueReusableCell({{ viewController.\(Keys.ViewController.Name) }}.TableViewCellPrototypes.{{ dequeueFunction.\(Keys.ViewController.DequeueFunction.ReuseIdentifier) }})\n" +
            
            "    }\n" +
            "\n" +
            
            "    func dequeueReusable{{ dequeueFunction.\(Keys.ViewController.DequeueFunction.ReuseIdentifier) }}(tableView : UITableView, forIndexPath indexPath : NSIndexPath) -> AnyObject {\n" +
            
            "        return tableView.dequeueReusableCell({{ viewController.\(Keys.ViewController.Name) }}.TableViewCellPrototypes.{{ dequeueFunction.\(Keys.ViewController.DequeueFunction.ReuseIdentifier) }}, forIndexPath : indexPath)\n" +
            
            "    }\n" +
            
            "{% endfor %}" +
        "{% endif %}"
    }

    static var sourceFile : String {
        return self.topFileComments + "\n" +
            "\n" +
            "import UIKit\n" +
            
// As soon as you parse multiple Storyboards this will break your project, for now we're always going to export it separately
//            "{% if \(Keys.TogetherSharedDefinitions) %}" +
//            "\n" +
//            "{{ \(Keys.TogetherSharedDefinitions) }}" +
//            "{% endif %}" +
            
            "{% if \(Keys.ViewControllers) %}" +
            "{% for viewController in \(Keys.ViewControllers) %}" +
            
            "\n" +
            "extension {{ viewController.\(Keys.ViewController.Name) }} {\n" +
            self.segueCases +
            self.performFunctions +
            self.tableViewCellPrototypes +
            self.dequeueFunctions +
            "}\n" +

            "{% endfor %}" +
            "{% endif %}"
    }

    
    static var sharedDefinitions : String {
        return self.sharedUIViewControllerDefinitions +
        "\n" +
        self.sharedUITableViewDefinitions
    }
    
    static var sharedUIViewControllerDefinitions : String {
        return "extension UIViewController {\n" +
            
            "    class Segue\n" +
            "    {\n" +
            "        let identifier : String\n" +
            "\n" +
            "        init(identifier : String) {\n" +
            "            self.identifier = identifier\n" +
            "        }\n" +
            "    }\n" +
            "\n" +
            
            "    func performSegue(segue : Segue, sender : AnyObject?) {\n" + // TODO: validate that we're calling from the correct instance of the VC class for classes in multiple instances in storyboards
            "        self.performSegueWithIdentifier(segue.identifier, sender: sender)\n" +
            "    }\n" +
            "}\n"
    }
    
    static var sharedUITableViewDefinitions : String {
        return "extension UITableView {\n" +
            
        "    class TableViewCellPrototype\n" +
        "    {\n" +
        "        let reuseIdentifier : String\n" +
        "\n" +
        "        init(reuseIdentifier : String) {\n" +
        "            self.reuseIdentifier = reuseIdentifier\n" +
        "        }\n" +
        "    }\n" +
        "\n" +
            
        "    func dequeueReusableCell(cellPrototype : TableViewCellPrototype) -> AnyObject? {\n" +
        "        return self.dequeueReusableCellWithIdentifier(cellPrototype.reuseIdentifier)\n" +
        "    }\n" +
        "\n" +
            
        "    func dequeueReusableCell(cellPrototype : TableViewCellPrototype, forIndexPath indexPath : NSIndexPath) -> AnyObject {\n" +
        "        return self.dequeueReusableCellWithIdentifier(cellPrototype.reuseIdentifier, forIndexPath: indexPath)\n" +
        "    }\n" +
            
        "}\n"
    }
    
    static var sharedFile : String {
        return self.topFileComments + "\n" +
            "\n" +
            "import UIKit\n" +
            "\n" + 
            self.sharedDefinitions
    }
    
    required init() {
        super.init(templateString: DefaultTemplate.sourceFile)
    }
}
